"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WalkNode = void 0;
const defaults_1 = require("./defaults");
function getNormalizedType(val) {
    if (Array.isArray(val))
        return 'array';
    else if (typeof val === 'object')
        return 'object';
    return 'value';
}
class WalkNode {
    constructor(val, isRoot = false, isArrayMember = false, nodeType = 'value', rawType = 'undefined', executedCallbacks = [], keyInParent, parent) {
        this.val = val;
        this.isRoot = isRoot;
        this.isArrayMember = isArrayMember;
        this.nodeType = nodeType;
        this.rawType = rawType;
        this.executedCallbacks = executedCallbacks;
        this.keyInParent = keyInParent;
        this.parent = parent;
        this.id = ++WalkNode._idx;
    }
    static fromRoot(obj) {
        return new WalkNode(obj, true, false, 'object', typeof obj, [], undefined, undefined);
    }
    static fromObjectKey(parent, key, ctx) {
        return new WalkNode(parent.val[key], false, false, getNormalizedType(parent.val[key]), typeof parent.val[key], [], key, parent);
    }
    static fromArrayIndex(parent, index, ctx) {
        return new WalkNode(parent.val[index], false, true, getNormalizedType(parent.val[index]), typeof parent.val[index], [], index, parent);
    }
    getPath(pathFormat) {
        if (this.isRoot)
            return "";
        pathFormat = pathFormat || defaults_1.defaultPathFormat;
        return this.parent.getPath(pathFormat) + pathFormat(this.keyInParent.toString(), this.isArrayMember);
    }
    children(ctx) {
        if (typeof this._children !== 'undefined')
            return this._children;
        if (this.nodeType === 'array')
            this._children = this.val.map((_, idx) => WalkNode.fromArrayIndex(this, idx, ctx));
        else if (this.nodeType === 'object')
            this._children = Object.keys(this.val).map(key => WalkNode.fromObjectKey(this, key, ctx));
        else
            this._children = [];
        return this._children;
    }
    siblings(ctx) {
        if (!this.parent)
            return [];
        return this.parent.children(ctx).filter((c) => c.id !== this.id);
    }
}
exports.WalkNode = WalkNode;
