"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Break = exports.deepCopy = exports.findAll = exports.apply = exports.flatten = void 0;
const helpers_1 = require("./helpers");
const walk_1 = require("./walk");
function flatten(obj, key, onlyUnique) {
    //return array of values that match the key
    const arr = [];
    walk_1.default(obj, {
        callbacks: [{
                keyFilters: [key],
                callback: function (node) {
                    arr.push(node.val);
                }
            }]
    });
    return onlyUnique ? helpers_1.unique(arr) : arr;
}
exports.flatten = flatten;
function apply(obj, callback) {
    walk_1.default(obj, { callbacks: [{ callback }] });
}
exports.apply = apply;
function findAll(obj, value, typeConversion = false) {
    const comparison = typeConversion
        ? (a, b) => a == b
        : (a, b) => a === b;
    const matches = [];
    apply(obj, function (node) {
        if (comparison(node.val, value)) {
            matches.push(node.val);
        }
    });
    return matches;
}
exports.findAll = findAll;
function deepCopy(obj) {
    const newObj = {};
    const uuid = 'WALK:DEEP-COPY:DELIMITER';
    const format = (key) => uuid + key;
    walk_1.default(obj, {
        rootObjectCallbacks: false,
        callbacks: [{
                positionFilters: ['preWalk'],
                callback: function (node) {
                    switch (node.nodeType) {
                        case 'array':
                            helpers_1.updateObjectViaPathString(newObj, [], node.getPath(format), uuid);
                            break;
                        case 'object':
                            helpers_1.updateObjectViaPathString(newObj, {}, node.getPath(format), uuid);
                            break;
                        case 'value':
                            helpers_1.updateObjectViaPathString(newObj, node.val, node.getPath(format), uuid);
                            break;
                    }
                }
            }]
    });
    return newObj;
}
exports.deepCopy = deepCopy;
class Break extends Error {
    constructor(message = '') {
        super(message);
        this.name = "Break";
    }
}
exports.Break = Break;
