"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const defaults_1 = require("./defaults");
const helpers_1 = require("./helpers");
const utils_1 = require("./utils");
const node_1 = require("./node");
function matchCallbacks(node, position, ctx) {
    if (!ctx.config.runCallbacks)
        return [];
    if (node.isRoot && !ctx.config.rootObjectCallbacks)
        return [];
    const callbacks = ctx.callbacksByPosition[position];
    if (typeof callbacks == 'undefined') {
        return [];
    }
    return callbacks
        .filter(cb => { var _a, _b; return ((_b = (_a = cb.nodeTypeFilters) === null || _a === void 0 ? void 0 : _a.indexOf(node.nodeType)) !== null && _b !== void 0 ? _b : 1) !== -1; })
        .filter(cb => typeof cb.keyFilters === 'undefined'
        || (typeof node.keyInParent !== 'number' && cb.keyFilters.indexOf(node.keyInParent) !== -1));
}
function validateVisitation(node, ctx) {
    const seen = ctx.seenObjects.indexOf(node.val) !== -1;
    if (seen && ctx.config.graphMode === 'graph')
        return false;
    if (seen && ctx.config.graphMode === 'finiteTree')
        throw "The object violates the defined structure. Override 'graphMode' in the config to allow parsing different object structures.";
    ctx.seenObjects.push(node.val);
    return true;
}
function execCallbacks(callbacks, node) {
    callbacks.forEach(cb => {
        cb.callback(node);
        node.executedCallbacks.push(cb);
    });
}
function process(node, mode, ctx) {
    if ((node.nodeType !== 'value') && !validateVisitation(node, ctx))
        return [];
    execCallbacks(matchCallbacks(node, 'preWalk', ctx), node);
    const queue = [];
    const traverse = mode === 'breadth'
        ? (child) => queue.push(child)
        : (child) => process(child, 'depth', ctx);
    node.children(ctx).forEach(traverse);
    execCallbacks(matchCallbacks(node, 'postWalk', ctx), node);
    return queue;
}
function depthTraverse(root, ctx) {
    process(root, 'depth', ctx);
}
function breadthTraverse(root, ctx) {
    let queue = [root];
    do
        queue = queue.concat(process(queue.shift(), 'breadth', ctx));
    while (queue.length > 0);
}
function buildContext(config) {
    const ctx = defaults_1.buildDefaultContext(config);
    ctx.config.callbacks.forEach(cb => {
        const callback = Object.assign(Object.assign({}, cb), { executionOrder: typeof cb.executionOrder == 'undefined' ? 0 : cb.executionOrder, positionFilters: typeof cb.positionFilters == 'undefined' || cb.positionFilters.length < 1 ? [defaults_1.defaultCallbackPosition] :
                cb.positionFilters });
        callback.positionFilters.forEach(position => {
            if (!(position in ctx.callbacksByPosition))
                ctx.callbacksByPosition[position] = [];
            ctx.callbacksByPosition[position].push(callback);
        });
    });
    for (const key in ctx.callbacksByPosition) {
        ctx.callbacksByPosition[key] = ctx.callbacksByPosition[key].sort(helpers_1.executionOrderSort);
    }
    return ctx;
}
const walk = (obj, config) => {
    const context = buildContext(config);
    const rootNode = node_1.WalkNode.fromRoot(obj);
    const fn = context.config.traversalMode === 'depth'
        ? depthTraverse
        : breadthTraverse;
    try {
        fn(rootNode, context);
    }
    catch (err) {
        if (!(err instanceof utils_1.Break))
            throw err;
    }
    return context;
};
exports.default = walk;
